/* ============================================================================
 * File Name: ADC-Filter-VDAC_01c.c
 *
 * Description:
 *   Demo project showing DelSig_ADC -> Filter -> VDAC data streaming using DMA
 *   This version is modified from the original project by Todd Dust
 *   to transfer both posivive and negative values of input Sine correctly.
 *
 *   To do so the DMA_1 channel (Filter-> VDAC8) is modified to include
 *   two Transfer Descriptors (TD[0] & TD[1]), which are executed immediately
 *   one after another, and extra logic to convert signed output from the Filter to
 *   unsigned value for VDAC8 (int8->uint8).
 *
 * Credits:
 *   based on the original project by Todd Dust
 *    https://www.compuphase.com/electronics/debouncing.htm
 * 
 *   and extra hint by <Pavloven>
 *
 * Note:
 *   This project uses several custom Community components and libraries,
 *   which can be omitted if an external signal generator is available.
 * 
 *   1. WaveGen8 (RAM-DMA-VDAC8 wave generator, DRAFT version is included in the project)
 *   2. QuadDecoder_SW: https://community.cypress.com/thread/30654
 *   3. Annotation Library: https://community.cypress.com/thread/48049
 *   4. DDS32: https://community.cypress.com/message/158566 (included in the project)  
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#include <project.h>


#define BlinkLED(); {Pin_LED_Write(1); CyDelayUs(130); Pin_LED_Write(0);} // blink LED indicator

//===========================================
// Function prototypes
//===========================================
void DMA_Config(void);
void DMA_1_Config(void);


void Init()
{
    // Enable Global Interrupts
    CYGlobalIntEnable;
    
    // Start all components used on schematic
    ADC_1_IRQ_Start();
    ADC_1_Start();
    ADC_1_StartConvert();
    
    VDAC_1_Start();
    Opamp_1_Start();
    
    Opamp_Start();
    Filter_Start();

    // User-implemented function to set-up DMA 
    DMA_Config();
    DMA_1_Config();
    
    
    //Wave genrator with rotary encoder->
    WaveGen_1_CreateSineWave_B(255, 0); // fill Buffer with default values (ampl, phase) 
    WaveGen_1_Start();                  // start wave generator
    VDAC_2_Start();
    Opamp_2_Start();
    
    Decoder_1_Start();
    DDS_1_Start();
    DDS_1_SetFrequency(Decoder_1_Position);
}

int main()
{
    Init();

    for(;;)
    {
        //CyPmAltAct(PM_SLEEP_TIME_NONE, PM_SLEEP_SRC_PICU);
        
        if (Decoder_1_PositionChanged)              // encoder position changed
        { 
            DDS_1_SetFrequency(Decoder_1_Position);
            //BlinkLED();   // debugging...
        }
        
    }
    
} /* End of main */


/*******************************************************************************
* Function Name: DMA_Config
********************************************************************************
*
* Summary:
*  Initializes and sets up DMA for use (generated by DMA Wizard)
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void DMA_Config(void)
{
    #define REQUEST_PER_BURST        (1u)
    #define BYTES_PER_BURST          (1u)
    #define UPPER_SRC_ADDRESS        CYDEV_PERIPH_BASE
    #define UPPER_DEST_ADDRESS       CYDEV_PERIPH_BASE

    // Declare variable to hold the handle for DMA channel /
    uint8 channelHandle;

    // Declare DMA Transaction Descriptor for memory transfer into Filter Channel./
    uint8 tdChanA;

    // Configure the DMA to Transfer the data in 1 burst with individual trigger for each burst.
    channelHandle = DMA_DmaInitialize(BYTES_PER_BURST, REQUEST_PER_BURST, HI16(UPPER_SRC_ADDRESS), HI16(UPPER_DEST_ADDRESS));

    // This function allocates a TD for use with an initialized DMA channel
    tdChanA = CyDmaTdAllocate();

    // Configure the tdChanA to transfer 1 byte with no next TD
    CyDmaTdSetConfiguration(tdChanA, 1u, DMA_INVALID_TD, 0u);

    // Set the source address as ADC_DelSig and the destination as Filter Channel A
    CyDmaTdSetAddress(tdChanA, LO16((uint32)ADC_1_DEC_SAMP_PTR), LO16((uint32)Filter_STAGEAH_PTR));

    // Set tdChanA to be the initial TD associated with channelHandle 
    CyDmaChSetInitialTd(channelHandle, tdChanA);

    // Enable the DMA channel represented by channelHandle and preserve the TD
    CyDmaChEnable(channelHandle, 1u);
}


//======================================================================================
// Configures DMA_1 transfer Filter->ControlReg->StatusReg->VDAC8 
// DMA_1 has two TDs:
// TD[0] transfers signed (int8) data from Filter to ControlReg
// TD[1] transfers unsigned (uint8) data from StatusReg to VDAC8
//======================================================================================
void DMA_1_Config(void)
{
    // Variable declarations for DMA_1
    uint8 DMA_1_Chan;
    uint8 DMA_1_TD[2];

    #define TRANSFER_COUNT1 (1u)
    #define TRANSFER_COUNT2 (1u)

    // DMA Configuration for DMA_1 
    #define DMA_1_BYTES_PER_BURST 1
    #define DMA_1_REQUEST_PER_BURST 1
    #define DMA_1_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_1_DST_BASE (CYDEV_PERIPH_BASE)

    DMA_1_Chan = DMA_1_DmaInitialize(DMA_1_BYTES_PER_BURST, DMA_1_REQUEST_PER_BURST, HI16(DMA_1_SRC_BASE), HI16(DMA_1_DST_BASE));
    DMA_1_TD[0] = CyDmaTdAllocate();
    DMA_1_TD[1] = CyDmaTdAllocate();
    
    CyDmaTdSetConfiguration(DMA_1_TD[0], TRANSFER_COUNT1, DMA_1_TD[1], 0); // next is TD[1]
    CyDmaTdSetConfiguration(DMA_1_TD[1], TRANSFER_COUNT2, DMA_1_TD[0], 0); // return to TD[0]
    //CyDmaTdSetConfiguration(DMA_1_TD[0], TRANSFER_COUNT1, DMA_1_TD[1], DMA_1__TD_TERMOUT_EN); // next is TD[1]
    //CyDmaTdSetConfiguration(DMA_1_TD[1], TRANSFER_COUNT2, DMA_1_TD[0], DMA_1__TD_TERMOUT_EN); // return to TD[0]
    
    // set Target 
    CyDmaTdSetAddress(DMA_1_TD[0], LO16((uint32)Filter_HOLDAH_PTR), LO16((uint32)CReg_1_Control_PTR)); // Filter -> CReg
	CyDmaTdSetAddress(DMA_1_TD[1], LO16((uint32)SReg_1_Status_PTR), LO16((uint32)VDAC_1_Data_PTR));    // SReg   -> VDAC

    CyDmaChSetInitialTd(DMA_1_Chan, DMA_1_TD[0]);
    CyDmaChEnable(DMA_1_Chan, 1);
}


/*
//======================================================================================
// This is original routine from the Cypress demo
// it configures DMA_1 direct transfer from Filter to VDAC8 
// DMA_1 has single TD:
// TD[0] transfers signed (int8) data from Filter to VDAC8
//======================================================================================
void DMA_1_Config(void)
{
    // Variable declarations for DMA_1
    uint8 DMA_1_Chan;
    uint8 DMA_1_TD[1];

    #define TRANSFER_COUNT1 (1u)

    // DMA Configuration for DMA_1 
    #define DMA_1_BYTES_PER_BURST 1
    #define DMA_1_REQUEST_PER_BURST 1
    #define DMA_1_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_1_DST_BASE (CYDEV_PERIPH_BASE)

    DMA_1_Chan = DMA_1_DmaInitialize(DMA_1_BYTES_PER_BURST, DMA_1_REQUEST_PER_BURST, HI16(DMA_1_SRC_BASE), HI16(DMA_1_DST_BASE));
    DMA_1_TD[0] = CyDmaTdAllocate();
    
    CyDmaTdSetConfiguration(DMA_1_TD[0], TRANSFER_COUNT1, DMA_1_TD[0], 0);

    CyDmaTdSetAddress(DMA_1_TD[0], LO16((uint32)Filter_HOLDAH_PTR), LO16((uint32)VDAC_1_Data_PTR)); //Filter->VDAC

    CyDmaChSetInitialTd(DMA_1_Chan, DMA_1_TD[0]);
    CyDmaChEnable(DMA_1_Chan, 1);
}
*/

/* [] END OF FILE */
